/*
************************************************************************
*
*   ExLabel.c - labelling commands
*
*   Copyright (c) 1994
*
*   ETH Zuerich
*   Institut fuer Molekularbiologie und Biophysik
*   ETH-Hoenggerberg
*   CH-8093 Zuerich
*
*   SPECTROSPIN AG
*   Industriestr. 26
*   CH-8117 Faellanden
*
*   All Rights Reserved
*
*   Date of last modification : 94/06/02
*   Pathname of SCCS file     : /sgiext/molmol/src/cmdattr/SCCS/s.ExLabel.c
*   SCCS identification       : 1.1
*
************************************************************************
*/

#include <cmd_attr.h>

#include <stdio.h>
#include <string.h>
#include <memory.h>

#include <arg.h>
#include <data_hand.h>
#include <attr_struc.h>
#include <attr_mng.h>
#include <graph_draw.h>

static char AtomFormat[LABEL_FORMAT_SIZE] = "";
static int ChoiceDistLabel = 0;

static void
setAtomLabel(DhAtomP atomP, void *clientData)
{
  AttrP attrP;
  struct AttrS attr;

  attrP = DhAtomGetAttr(atomP);
  AttrCopy(&attr, attrP);
  AttrReturn(attrP);
  (void) memcpy(attr.labelFormat, AtomFormat, sizeof(AtomFormat));
  DhAtomSetAttr(atomP, AttrGet(&attr));
}

static void
setDistLabel(DhDistP distP, DhAtomP atom1P, DhAtomP atom2P, void *clientData)
{
  DistLabel style = *(DistLabel *) clientData;
  AttrP attrP;
  struct AttrS attr;

  attrP = DhDistGetAttr(distP);
  AttrCopy(&attr, attrP);
  AttrReturn(attrP);
  attr.distLabel = style;
  DhDistSetAttr(distP, AttrGet(&attr));
}

#define ARG_NUM 1

ErrCode
ExLabelAtom(char *cmd)
{
  ArgDescr arg[ARG_NUM];
  ErrCode errCode;
  int len;

  arg[0].type = AT_STR;

  ArgInit(arg, ARG_NUM);

  arg[0].prompt = "Label Format";

  errCode = ArgGet(arg, ARG_NUM);
  if (errCode != EC_OK) {
    ArgCleanup(arg, ARG_NUM);
    return errCode;
  }

  len = DStrLen(arg[0].v.strVal);
  if (len >= sizeof(AtomFormat))
    len = sizeof(AtomFormat) - 1;

  (void) memcpy(AtomFormat, DStrToStr(arg[0].v.strVal), len);
  (void) memset(AtomFormat + len, 0, sizeof(AtomFormat) - len);

  ArgCleanup(arg, ARG_NUM);

  DhApplyAtom(PropGetRef(PROP_SELECTED, FALSE), setAtomLabel, NULL);
  GraphMolChanged(PROP_SELECTED);

  GraphRedrawNeeded();

  return EC_OK;
}

ErrCode
ExLabelDist(char *cmd)
{
  ArgDescr arg[ARG_NUM];
  EnumEntryDescr enumEntry[4];
  ErrCode errCode;
  DistLabel style;

  arg[0].type = AT_ENUM;

  ArgInit(arg, ARG_NUM);

  enumEntry[0].str = "invisible";
  enumEntry[0].onOff = FALSE;
  enumEntry[1].str = "val";
  enumEntry[1].onOff = FALSE;
  enumEntry[2].str = "limit";
  enumEntry[2].onOff = FALSE;
  enumEntry[3].str = "viol";
  enumEntry[3].onOff = FALSE;

  enumEntry[ChoiceDistLabel].onOff = TRUE;

  arg[0].prompt = "Dist Label";
  arg[0].u.enumD.entryP = enumEntry;
  arg[0].u.enumD.n = 4;
  arg[0].v.intVal = ChoiceDistLabel;

  errCode = ArgGet(arg, ARG_NUM);
  if (errCode != EC_OK) {
    ArgCleanup(arg, ARG_NUM);
    return errCode;
  }

  ChoiceDistLabel = arg[0].v.intVal;

  if (ChoiceDistLabel == 0)
    style = DL_INVISIBLE;
  else if (ChoiceDistLabel == 1)
    style = DL_VAL;
  else if (ChoiceDistLabel == 2)
    style = DL_LIMIT;
  else
    style = DL_VIOL;

  DhApplyDist(PropGetRef(PROP_SELECTED, FALSE), setDistLabel, &style);

  GraphMolChanged(PROP_SELECTED);
  GraphRedrawNeeded();

  return EC_OK;
}
