/*
************************************************************************
*
*   ExBuild.c - NewMol, AddRes and ChangeRes commands
*
*   Copyright (c) 1994-96
*
*   ETH Zuerich
*   Institut fuer Molekularbiologie und Biophysik
*   ETH-Hoenggerberg
*   CH-8093 Zuerich
*
*   SPECTROSPIN AG
*   Industriestr. 26
*   CH-8117 Faellanden
*
*   All Rights Reserved
*
*   Date of last modification : 96/04/12
*   Pathname of SCCS file     : /sgiext/molmol/src/cmdstruc/SCCS/s.ExBuild.c
*   SCCS identification       : 1.2
*
************************************************************************
*/

#include <cmd_struc.h>

#include <stdio.h>

#include <arg.h>
#include <data_hand.h>
#include <data_sel.h>
#include <graph_draw.h>

#define ARG_NUM 1

static int CurrKind = 0;

ErrCode
ExNewMol(char *cmd)
{
  ArgDescr arg[ARG_NUM];
  ErrCode errCode;
  DhMolP molP;

  arg[0].type = AT_STR;

  ArgInit(arg, ARG_NUM);

  arg[0].prompt = "Molecule Name";

  errCode = ArgGet(arg, ARG_NUM);
  if (errCode != EC_OK) {
    ArgCleanup(arg, ARG_NUM);
    return errCode;
  }

  DeselAllMol();

  molP = DhMolNew();
  DhMolSetName(molP, arg[0].v.strVal);

  ArgCleanup(arg, ARG_NUM);

  DhMolSetPropStand(molP);
  GraphMolAdd(molP);

  return EC_OK;
}

#undef ARG_NUM
#define ARG_NUM 1

ErrCode
ExAddRes(char *cmd)
{
  ArgDescr arg[ARG_NUM];
  ErrCode errCode;
  int selNo;
  DhMolP molP;
  DhResDefP resDefP;
  DhResP resP;

  arg[0].type = AT_STR;

  ArgInit(arg, ARG_NUM);

  arg[0].prompt = "Residue Type";

  errCode = ArgGet(arg, ARG_NUM);
  if (errCode != EC_OK) {
    ArgCleanup(arg, ARG_NUM);
    return errCode;
  }

  resDefP = DhResDefGet(arg[0].v.strVal);
  ArgCleanup(arg, ARG_NUM);

  selNo = SelMolGet(&molP, 1);
  if (selNo != 1) {
    CipSetError("exactly 1 molecule must be selected");
    return EC_ERROR;
  }

  if (resDefP == NULL) {
    CipSetError("unknown residue type");
    return EC_ERROR;
  }

  resP = DhResNew(molP, resDefP);
  DhResStandGeom(resP);
  DhResDockPrev(resP);

  DhResSetPropStand(resP);
  GraphMolChanged(PROP_SELECTED);
  GraphRedrawNeeded();

  return EC_OK;
}

static void
resMutate(DhResP resP, void *clientData)
{
  DhResDefP resDefP = clientData;

  DhResMutate(resP, resDefP, CurrKind);
  DhResSetPropStand(resP);
}

#undef ARG_NUM
#define ARG_NUM 2
#define KIND_NUM 2

ErrCode
ExChangeRes(char *cmd)
{
  ArgDescr arg[ARG_NUM];
  EnumEntryDescr enumEntry[KIND_NUM];
  ErrCode errCode;
  DhResDefP resDefP;

  arg[0].type = AT_ENUM;
  arg[1].type = AT_STR;

  ArgInit(arg, ARG_NUM);

  enumEntry[0].str = "angles";
  enumEntry[0].onOff = FALSE;
  enumEntry[1].str = "atoms";
  enumEntry[1].onOff = FALSE;

  enumEntry[CurrKind].onOff = TRUE;

  arg[0].prompt = "Keep";
  arg[0].u.enumD.entryP = enumEntry;
  arg[0].u.enumD.n = KIND_NUM;
  arg[0].v.intVal = CurrKind;

  arg[1].prompt = "Residue Type";

  errCode = ArgGet(arg, ARG_NUM);
  if (errCode != EC_OK) {
    ArgCleanup(arg, ARG_NUM);
    return errCode;
  }

  CurrKind = arg[0].v.intVal;
  resDefP = DhResDefGet(arg[1].v.strVal);

  ArgCleanup(arg, ARG_NUM);

  if (resDefP == NULL) {
    CipSetError("unknown residue type");
    return EC_ERROR;
  }

  DhApplyRes(PropGetRef(PROP_SELECTED, FALSE), resMutate, resDefP);

  GraphMolChanged(PROP_SELECTED);
  GraphRedrawNeeded();

  return EC_OK;
}
